@@ -1,30 +1,20 @@
-/*************************************************************************************
- *  Copyright (C) 2012 by Alejandro Fiestas Olivares <afiestas@kde.org>              *
- *                                                                                   *
- *  This library is free software; you can redistribute it and/or                    *
- *  modify it under the terms of the GNU Lesser General Public                       *
- *  License as published by the Free Software Foundation; either                     *
- *  version 2.1 of the License, or (at your option) any later version.               *
- *                                                                                   *
- *  This library is distributed in the hope that it will be useful,                  *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                *
- *  Lesser General Public License for more details.                                  *
- *                                                                                   *
- *  You should have received a copy of the GNU Lesser General Public                 *
- *  License along with this library; if not, write to the Free Software              *
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA       *
- *************************************************************************************/
+/*
+ *  SPDX-FileCopyrightText: 2012 Alejandro Fiestas Olivares <afiestas@kde.org>
+ *
+ *  SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #include "xrandr11.h"
+
 #include "../xcbeventlistener.h"
+#include "../xcbwrapper.h"
 
 #include "config.h"
-#include "edid.h"
+#include "mode.h"
 #include "output.h"
+#include "screen.h"
 
 #include <xcb/randr.h>
-#include <xcb/xcb.h>
 
 #include <QDebug>
 #include <QString>
@@ -121,7 +111,7 @@ KScreen::ConfigPtr XRandR11::config() const
     output->setEnabled(true);
     output->setName(QStringLiteral("Default"));
     output->setPos(QPoint(0, 0));
-    output->setPrimary(true);
+    output->setPriority(1);
     output->setRotation((KScreen::Output::Rotation)info->rotation);
     output->setSizeMm(QSize(xcbScreen->width_in_millimeters, xcbScreen->height_in_millimeters));
 
